from mayan.apps.documents.tests.base import GenericDocumentViewTestCase
from mayan.apps.testing.tests.base import GenericViewTestCase

from ..events import event_workflow_template_edited
from ..models import WorkflowTransition
from ..permissions import (
    permission_workflow_template_edit, permission_workflow_template_view
)

from .literals import TEST_WORKFLOW_TEMPLATE_TRANSITION_LABEL
from .mixins.workflow_instance_mixins import WorkflowInstanceViewTestMixin
from .mixins.workflow_template_mixins import (
    WorkflowTemplateTestMixin, WorkflowTemplateViewTestMixin
)
from .mixins.workflow_template_transition_mixins import (
    WorkflowTransitionEventViewTestMixin, WorkflowTransitionFieldTestMixin,
    WorkflowTransitionFieldViewTestMixin, WorkflowTransitionViewTestMixin
)


class WorkflowTransitionViewTestCase(
    WorkflowTemplateTestMixin, WorkflowTemplateViewTestMixin,
    WorkflowTransitionViewTestMixin, GenericViewTestCase
):
    def setUp(self):
        super().setUp()
        self._create_test_workflow_template()
        self._create_test_workflow_template_state()
        self._create_test_workflow_template_state()

    def test_workflow_template_transition_create_view_no_permission(self):
        self._clear_events()

        response = self._request_test_workflow_template_transition_create_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(WorkflowTransition.objects.count(), 0)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_create_view_with_access(self):
        self.grant_access(
            obj=self.test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_transition_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(WorkflowTransition.objects.count(), 1)
        self.assertEqual(
            WorkflowTransition.objects.all()[0].label,
            TEST_WORKFLOW_TEMPLATE_TRANSITION_LABEL
        )
        self.assertEqual(
            WorkflowTransition.objects.all()[0].origin_state,
            self.test_workflow_template_states[0]
        )
        self.assertEqual(
            WorkflowTransition.objects.all()[0].destination_state,
            self.test_workflow_template_states[1]
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(
            events[0].action_object, self.test_workflow_template_transition
        )
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self.test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_workflow_template_transition_delete_view_no_permission(self):
        self._create_test_workflow_template_transition()

        self._clear_events()

        response = self._request_test_workflow_template_transition_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertTrue(
            self.test_workflow_template_transition in WorkflowTransition.objects.all()
        )
        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_delete_view_with_access(self):
        self._create_test_workflow_template_transition()

        self.grant_access(
            obj=self.test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_transition_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertFalse(
            self.test_workflow_template_transition in WorkflowTransition.objects.all()
        )
        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self.test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_workflow_template_transition_edit_view_no_permission(self):
        self._create_test_workflow_template_transition()

        test_workflow_template_transition_label = self.test_workflow_template_transition.label

        self._clear_events()

        response = self._request_test_workflow_template_transition_edit_view()
        self.assertEqual(response.status_code, 404)

        self.test_workflow_template_transition.refresh_from_db()
        self.assertEqual(
            self.test_workflow_template_transition.label,
            test_workflow_template_transition_label
        )
        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_edit_view_with_access(self):
        self._create_test_workflow_template_transition()

        test_workflow_template_transition_label = self.test_workflow_template_transition.label

        self.grant_access(
            obj=self.test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_transition_edit_view()
        self.assertEqual(response.status_code, 302)

        self.test_workflow_template_transition.refresh_from_db()
        self.assertNotEqual(
            self.test_workflow_template_transition.label,
            test_workflow_template_transition_label
        )
        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(
            events[0].action_object, self.test_workflow_template_transition
        )
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self.test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_workflow_template_transition_list_view_no_permission(self):
        self._create_test_workflow_template_transition()

        self._clear_events()

        response = self._request_test_workflow_template_transition_list_view()
        self.assertNotContains(
            response=response,
            text=self.test_workflow_template_transition.label,
            status_code=404
        )
        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_list_view_with_access(self):
        self._create_test_workflow_template_transition()

        self.grant_access(
            obj=self.test_workflow_template,
            permission=permission_workflow_template_view
        )

        self._clear_events()

        response = self._request_test_workflow_template_transition_list_view()
        self.assertContains(
            response=response,
            text=self.test_workflow_template_transition.label,
            status_code=200
        )
        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class WorkflowTransitionEventViewTestCase(
    WorkflowInstanceViewTestMixin, WorkflowTemplateTestMixin,
    WorkflowTransitionEventViewTestMixin, GenericDocumentViewTestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_workflow_template()
        self.test_workflow_template.document_types.add(
            self.test_document_type
        )
        self._create_test_workflow_template_state()
        self._create_test_workflow_template_state()
        self._create_test_workflow_template_transition()
        self._create_test_workflow_template_transition()
        self._create_test_document_stub()
        self.test_workflow_instance = self.test_document.workflows.first()

    def test_workflow_template_transition_event_list_view_no_permission(self):
        self._clear_events()

        response = self._request_test_workflow_template_transition_event_list_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_event_list_view_with_access(self):
        self.grant_access(
            obj=self.test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_transition_event_list_view()
        self.assertEqual(response.status_code, 200)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class WorkflowTransitionFieldViewTestCase(
    WorkflowTemplateTestMixin, WorkflowTransitionFieldTestMixin,
    WorkflowTransitionFieldViewTestMixin, WorkflowTransitionViewTestMixin,
    GenericViewTestCase
):
    def setUp(self):
        super().setUp()
        self._create_test_workflow_template()
        self._create_test_workflow_template_state()
        self._create_test_workflow_template_state()
        self._create_test_workflow_template_transition()

    def test_workflow_template_transition_field_create_view_no_permission(self):
        workflow_template_transition_field_count = self.test_workflow_template_transition.fields.count()

        self._clear_events()

        response = self._request_workflow_template_transition_field_create_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self.test_workflow_template_transition.fields.count(),
            workflow_template_transition_field_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_field_create_view_with_access(self):
        workflow_template_transition_field_count = self.test_workflow_template_transition.fields.count()
        self.grant_access(
            obj=self.test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_workflow_template_transition_field_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_workflow_template_transition.fields.count(),
            workflow_template_transition_field_count + 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(
            events[0].action_object,
            self.test_workflow_template_transition_field
        )
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self.test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_workflow_template_transition_field_delete_view_no_permission(self):
        self._create_test_workflow_template_transition_field()
        workflow_template_transition_field_count = self.test_workflow_template_transition.fields.count()

        self._clear_events()

        response = self._request_workflow_template_transition_field_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self.test_workflow_template_transition.fields.count(),
            workflow_template_transition_field_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_field_delete_view_with_access(self):
        self._create_test_workflow_template_transition_field()
        workflow_template_transition_field_count = self.test_workflow_template_transition.fields.count()

        self.grant_access(
            obj=self.test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_workflow_template_transition_field_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_workflow_template_transition.fields.count(),
            workflow_template_transition_field_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self.test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_workflow_template_transition_field_edit_view_no_permission(self):
        self._create_test_workflow_template_transition_field()
        workflow_template_transition_field_label = self.test_workflow_template_transition_field.label

        self._clear_events()

        response = self._request_workflow_template_transition_field_edit_view()
        self.assertEqual(response.status_code, 404)

        self.test_workflow_template_transition_field.refresh_from_db()
        self.assertEqual(
            workflow_template_transition_field_label,
            self.test_workflow_template_transition_field.label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_field_edit_view_with_access(self):
        self._create_test_workflow_template_transition_field()
        workflow_template_transition_field_label = self.test_workflow_template_transition_field.label

        self.grant_access(
            obj=self.test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_workflow_template_transition_field_edit_view()
        self.assertEqual(response.status_code, 302)

        self.test_workflow_template_transition_field.refresh_from_db()
        self.assertNotEqual(
            workflow_template_transition_field_label,
            self.test_workflow_template_transition_field.label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(
            events[0].action_object,
            self.test_workflow_template_transition_field
        )
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self.test_workflow_template)
        self.assertEqual(events[0].verb, event_workflow_template_edited.id)

    def test_workflow_template_transition_field_list_view_no_permission(self):
        self._create_test_workflow_template_transition_field()

        self._clear_events()

        response = self._request_test_workflow_template_transition_field_list_view()
        self.assertNotContains(
            response=response,
            text=self.test_workflow_template_transition_field.label,
            status_code=404
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_workflow_template_transition_field_list_view_with_access(self):
        self._create_test_workflow_template_transition_field()
        self.grant_access(
            obj=self.test_workflow_template,
            permission=permission_workflow_template_edit
        )

        self._clear_events()

        response = self._request_test_workflow_template_transition_field_list_view()
        self.assertContains(
            response=response,
            text=self.test_workflow_template_transition_field.label,
            status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from rest_framework import serializers, status

from ..api_view_mixins import ExternalObjectAPIViewMixin
from ..generics import RetrieveAPIView

from .base import BaseAPITestCase
from .mixins import APIUserTestCaseMixin, TestAPIViewTestCaseMixin


class ExternalObjectAPIViewMixinTestCase(
    APIUserTestCaseMixin, TestAPIViewTestCaseMixin, BaseAPITestCase
):
    auto_add_test_view = True
    auto_create_test_object = True
    auto_create_test_object_permission = True
    auto_login_user = False
    test_view_url = r'^test-view-url/(?P<test_object_id>\d+)/$'

    def _test_view_factory(self, test_object=None):
        class TestModelSerializer(serializers.Serializer):
            """Empty serializer."""

        class TestView(ExternalObjectAPIViewMixin, RetrieveAPIView):
            external_object_queryset = self.TestModel.objects.all()
            external_object_pk_url_kwarg = 'test_object_id'
            mayan_external_object_permissions = {
                'GET': (self.test_permission,)
            }
            serializer_class = TestModelSerializer

            def get_object(self):
                return None

        return TestView.as_view()

    def _request_test_api_view(self):
        return self.get(
            headers={
                'HTTP_AUTHORIZATION': 'Token {}'.format(
                    self._test_case_user_token
                )
            }, viewname='rest_api:{}'.format(self.test_view_name), kwargs={
                'test_object_id': self.test_object.pk,
            },
        )

    def test_mixin_using_token_authentication_with_access(self):
        self.grant_access(
            obj=self.test_object, permission=self.test_permission
        )

        response = self._request_test_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_mixin_using_token_authentication_no_permission(self):
        response = self._request_test_api_view()

        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)



from .tasks import (
    task_unverify_key_signatures,
    task_verify_missing_embedded_signature,
    task_verify_key_signatures
)


def handler_verify_missing_embedded_signature(sender, **kwargs):
    task_verify_missing_embedded_signature.delay()


def handler_unverify_key_signatures(sender, **kwargs):
    task_unverify_key_signatures.apply_async(
        kwargs=dict(key_id=kwargs['instance'].key_id)
    )


def handler_verify_key_signatures(sender, **kwargs):
    task_verify_key_signatures.apply_async(
        kwargs=dict(key_pk=kwargs['instance'].pk)
    )



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('document_states', '0011_auto_20180315_0029'),
    ]

    operations = [
        migrations.AlterField(
            model_name='workflowstateaction',
            name='label',
            field=models.CharField(
                help_text='A simple identifier for this action.',
                max_length=255, verbose_name='Label'
            ),
        ),
    ]



from mayan.apps.common.tests.mixins import ObjectCopyTestMixin
from mayan.apps.documents.tests.mixins.document_mixins import DocumentTestMixin
from mayan.apps.testing.tests.base import BaseTestCase

from .mixins import MetadataTypeTestMixin


class MetadataTypeCopyTestCase(
    DocumentTestMixin, MetadataTypeTestMixin, ObjectCopyTestMixin, BaseTestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_metadata_type()
        self.test_metadata_type.document_types.create(
            document_type=self.test_document_type
        )
        self.test_object = self.test_metadata_type



import logging

from mayan.apps.rest_api import generics

from ..models.trashed_document_models import TrashedDocument
from ..permissions import (
    permission_document_view, permission_trashed_document_delete,
    permission_trashed_document_restore
)
from ..serializers.trashed_document_serializers import TrashedDocumentSerializer

logger = logging.getLogger(name=__name__)


class APITrashedDocumentDetailView(generics.RetrieveDestroyAPIView):
    """
    Returns the selected trashed document details.
    delete: Delete the trashed document.
    get: Retreive the details of the trashed document.
    """
    lookup_url_kwarg = 'document_id'
    mayan_object_permissions = {
        'DELETE': (permission_trashed_document_delete,),
        'GET': (permission_document_view,)
    }
    queryset = TrashedDocument.objects.all()
    serializer_class = TrashedDocumentSerializer

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user
        }


class APITrashedDocumentListView(generics.ListAPIView):
    """
    Returns a list of all the trashed documents.
    """
    mayan_object_permissions = {'GET': (permission_document_view,)}
    ordering_fields = ('label',)
    queryset = TrashedDocument.objects.all()
    serializer_class = TrashedDocumentSerializer


class APITrashedDocumentRestoreView(generics.ObjectActionAPIView):
    """
    post: Restore a trashed document.
    """
    lookup_url_kwarg = 'document_id'
    mayan_object_permissions = {
        'POST': (permission_trashed_document_restore,)
    }
    queryset = TrashedDocument.objects.all()

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user
        }

    def object_action(self, request, serializer):
        self.object.restore()



from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link
from mayan.apps.navigation.utils import get_content_type_kwargs_factory

from .icons import icon_object_error_list, icon_object_error_list_clear
from .permissions import permission_error_log_view


link_object_error_list = Link(
    icon=icon_object_error_list,
    kwargs=get_content_type_kwargs_factory(variable_name='resolved_object'),
    permissions=(permission_error_log_view,), text=_('Errors'),
    view='logging:object_error_list',
)
link_object_error_list_clear = Link(
    icon=icon_object_error_list_clear,
    kwargs=get_content_type_kwargs_factory(variable_name='resolved_object'),
    permissions=(permission_error_log_view,), text=_('Clear errors'),
    view='logging:object_error_list_clear',
)



from rest_framework import status

from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..events import (
    event_document_file_created, event_document_file_deleted,
    event_document_file_edited, event_document_file_downloaded,
    event_document_version_created, event_document_version_page_created
)
from ..permissions import (
    permission_document_file_delete, permission_document_file_download,
    permission_document_file_new, permission_document_file_view
)

from .mixins.document_mixins import DocumentTestMixin
from .mixins.document_file_mixins import (
    DocumentFileTestMixin, DocumentFileAPIViewTestMixin
)


class DocumentFileAPIViewTestCase(
    DocumentFileAPIViewTestMixin, DocumentTestMixin,
    DocumentFileTestMixin, BaseAPITestCase
):
    auto_upload_test_document = False

    def test_document_file_delete_api_view_no_permission(self):
        self._upload_test_document()
        self._upload_new_file()

        document_file_count = self.test_document.files.count()

        self._clear_events()

        response = self._request_test_document_file_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            self.test_document.files.count(), document_file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_delete_api_view_with_access(self):
        self._upload_test_document()
        self._upload_new_file()
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_file_delete
        )

        document_file_count = self.test_document.files.count()

        self._clear_events()

        response = self._request_test_document_file_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertEqual(
            self.test_document.files.count(), document_file_count - 1
        )

        self.assertEqual(
            self.test_document.files.first(), self.test_document.file_latest
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)
        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self.test_document)
        self.assertEqual(events[0].verb, event_document_file_deleted.id)

    def test_document_file_detail_api_view_no_permission(self):
        self._upload_test_document()

        self._clear_events()

        response = self._request_test_document_file_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_detail_api_view_with_access(self):
        self._upload_test_document()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_file_view
        )

        self._clear_events()

        response = self._request_test_document_file_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(
            response.data['checksum'], self.test_document.file_latest.checksum
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_download_api_view_no_permission(self):
        self._upload_test_document()

        self._clear_events()

        response = self._request_test_document_file_download_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_download_api_view_with_access(self):
        self._upload_test_document()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_file_download
        )

        self._clear_events()

        response = self._request_test_document_file_download_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        with self.test_document.file_latest.open() as file_object:
            self.assert_download_response(
                response=response, content=file_object.read(),
                filename=self.test_document.file_latest.filename,
                mime_type=self.test_document.file_latest.mimetype
            )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)
        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self.test_document_file)
        self.assertEqual(events[0].verb, event_document_file_downloaded.id)

    def test_document_file_list_api_view_no_permission(self):
        self._upload_test_document()

        self._clear_events()

        response = self._request_test_document_file_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_list_api_view_with_access(self):
        self._upload_test_document()

        self.grant_access(
            obj=self.test_document, permission=permission_document_file_view
        )

        self._clear_events()

        response = self._request_test_document_file_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(
            response.data['results'][0]['checksum'],
            self.test_document.file_latest.checksum
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_upload_api_view_no_permission(self):
        self._upload_test_document()

        document_file_count = self.test_document.files.count()

        self._clear_events()

        response = self._request_test_document_file_upload_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            self.test_document.files.count(), document_file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_upload_api_view_with_access(self):
        self._upload_test_document()

        self.grant_access(
            obj=self.test_document, permission=permission_document_file_new
        )

        document_file_count = self.test_document.files.count()

        self._clear_events()

        response = self._request_test_document_file_upload_api_view()
        self.assertEqual(response.status_code, status.HTTP_202_ACCEPTED)

        self.assertEqual(
            self.test_document.files.count(), document_file_count + 1
        )

        self.assertEqual(self.test_document.file_latest.exists(), True)
        self.assertEqual(self.test_document.file_latest.size, 17436)
        self.assertEqual(
            self.test_document.file_latest.mimetype, 'image/png'
        )
        self.assertEqual(self.test_document.file_latest.encoding, 'binary')
        self.assertEqual(
            self.test_document.file_latest.checksum,
            'efa10e6cc21f83078aaa94d5cbe51de67b51af706143bafc7fd6d4c02124879a'
        )
        self.assertEqual(self.test_document.pages.count(), 1)

        events = self._get_test_events()
        self.assertEqual(events.count(), 4)

        self.assertEqual(events[0].action_object, self.test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self.test_document_file)
        self.assertEqual(events[0].verb, event_document_file_created.id)

        self.assertEqual(events[1].action_object, self.test_document)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, self.test_document_file)
        self.assertEqual(events[1].verb, event_document_file_edited.id)

        self.test_document.refresh_from_db()
        test_document_version = self.test_document.versions.last()

        self.assertEqual(events[2].action_object, self.test_document)
        self.assertEqual(events[2].actor, self._test_case_user)
        self.assertEqual(events[2].target, test_document_version)
        self.assertEqual(events[2].verb, event_document_version_created.id)

        self.assertEqual(events[3].action_object, test_document_version)
        self.assertEqual(events[3].actor, self._test_case_user)
        self.assertEqual(events[3].target, test_document_version.pages.first())
        self.assertEqual(events[3].verb, event_document_version_page_created.id)



from django.conf.urls import url

from .views import (
    DocumentFileDriverListView, DocumentFileDriverEntryFileMetadataListView,
    DocumentFileSubmitView, DocumentTypeSettingsEditView,
    DocumentTypeSubmitView
)

urlpatterns = [
    url(
        regex=r'^documents/files/(?P<document_file_id>\d+)/drivers/$',
        name='document_file_driver_list', view=DocumentFileDriverListView.as_view()

    ),
    url(
        regex=r'^documents/files/(?P<document_file_id>\d+)/submit/$',
        name='document_file_submit', view=DocumentFileSubmitView.as_view()
    ),
    url(
        regex=r'^documents/files/multiple/submit/$',
        name='document_file_multiple_submit',
        view=DocumentFileSubmitView.as_view()
    ),
    url(
        regex=r'^documents/files/drivers/(?P<document_file_driver_id>\d+)/attributes/$',
        name='document_file_driver_file_metadata_list',
        view=DocumentFileDriverEntryFileMetadataListView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>\d+)/file_metadata/settings/$',
        name='document_type_settings',
        view=DocumentTypeSettingsEditView.as_view()
    ),
    url(
        regex=r'^document_types/submit/$', name='document_type_submit',
        view=DocumentTypeSubmitView.as_view()
    ),
]



from mayan.apps.documents.permissions import permission_document_view
from mayan.apps.documents.search import document_search
from mayan.apps.documents.tests.mixins.document_mixins import DocumentTestMixin
from mayan.apps.tags.tests.mixins import TagTestMixin
from mayan.apps.testing.tests.base import BaseTestCase

from ..classes import SearchBackend


class ScopedSearchTestCase(DocumentTestMixin, TagTestMixin, BaseTestCase):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self.search_backend = SearchBackend.get_instance()

        self._create_test_document_stub()
        self._create_test_tag()
        self._create_test_tag()

        self.test_tags[0].documents.add(self.test_document)
        self.test_tags[1].documents.add(self.test_document)

        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

    def test_AND_scope(self):
        query = {
            '__0_tags__label': self.test_tags[0].label,
            '__operator_0_1': 'AND_901',
            '__result': '901',
            '__1_tags__label': self.test_tags[1].label
        }
        queryset = self.search_backend.search(
            search_model=document_search, query_string=query,
            user=self._test_case_user
        )
        self.assertEqual(queryset.count(), 1)

        self.test_tags[1].documents.remove(self.test_document)

        query = {
            '__0_tags__label': self.test_tags[0].label,
            '__operator_0_1': 'AND_901',
            '__result': '901',
            '__1_tags__label': self.test_tags[1].label
        }
        queryset = self.search_backend.search(
            search_model=document_search, query_string=query,
            user=self._test_case_user
        )
        self.assertEqual(queryset.count(), 0)

    def test_OR_scope(self):
        query = {
            '__0_tags__label': self.test_tags[0].label,
            '__operator_0_1': 'OR_901',
            '__result': '901',
            '__1_tags__label': self.test_tags[1].label
        }
        queryset = self.search_backend.search(
            search_model=document_search, query_string=query,
            user=self._test_case_user
        )
        self.assertEqual(queryset.count(), 1)

    def test_letter_scopes(self):
        query = {
            '__a_tags__label': self.test_tags[0].label,
            '__operator_a_b': 'OR_c',
            '__result': 'c',
            '__b_tags__label': self.test_tags[1].label
        }
        queryset = self.search_backend.search(
            search_model=document_search, query_string=query,
            user=self._test_case_user
        )
        self.assertEqual(queryset.count(), 1)

    def test_multi_letter_scopes(self):
        query = {
            '__ab_tags__label': self.test_tags[0].label,
            '__operator_ab_bc': 'OR_cc',
            '__result': 'cc',
            '__bc_tags__label': self.test_tags[1].label
        }
        queryset = self.search_backend.search(
            search_model=document_search, query_string=query,
            user=self._test_case_user
        )
        self.assertEqual(queryset.count(), 1)



from django.apps import apps
from django.contrib import contenttypes
from django.core.exceptions import ImproperlyConfigured
from django.db import models
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _

import mptt

from mayan.apps.acls.classes import ModelPermission

from .links import link_object_copy
from .menus import menu_object
from .permissions import permission_object_copy


class ModelCopy:
    _registry = {}
    _lazy = {}

    @staticmethod
    def method_instance_copy(self, values=None):
        model_copy = ModelCopy.get(model=self._meta.model)
        return model_copy.copy(instance=self, values=values)

    @classmethod
    def add_fields_lazy(cls, model, **kwargs):
        cls._lazy.setdefault(model, [])
        cls._lazy[model].append(kwargs)

    @classmethod
    def get(cls, model):
        return cls._registry[model]

    def __init__(
        self, model, condition=None, bind_link=False, excludes=None, register_permission=False,
        extra_kwargs=None
    ):
        self.condition = condition
        self.excludes = excludes or {}
        self.extra_kwargs = extra_kwargs or {}
        self.fields = []
        self.fields_copy = []
        self.fields_foreign_keys = []
        self.fields_generic_related = []
        self.fields_many_to_many = []
        self.fields_reverse_related = []
        self.fields_many_to_many_reverse_related = []
        self.fields_related_one_to_one = []
        self.fields_unique = []

        self.model = model
        self.__class__._registry[model] = self
        model.add_to_class(
            name='copy_instance', value=ModelCopy.method_instance_copy
        )
        if bind_link:
            menu_object.bind_links(
                links=(link_object_copy,), sources=(model,), position=99
            )

        if register_permission:
            ModelPermission.register(
                model=model, permissions=(permission_object_copy,)
            )

        for entry in self.__class__._lazy.get(model, ()):
            self.add_fields(**entry)
            self.__class__._lazy.get(model).pop()

    def __str__(self):
        return force_text(s=self.label)

    def _evaluate_field_get_for_field(self, field, instance, value, values):
        context = {'instance': instance}
        context.update(values)

        field_value_gets = self.field_value_gets.get(field, None)
        if field_value_gets:
            related_model = self.model._meta.get_field(field).related_model or self.model._meta.get_field(field).model
            final_filter = {}
            for key, value in field_value_gets.items():
                final_filter[key] = value.format(**context)

            value = related_model._meta.default_manager.get(**final_filter)

        field_value_templates = self.field_value_templates.get(field, None)
        if field_value_templates:
            value = field_value_templates.format(**context) or None

        return value

    def add_fields(
        self, field_names, field_value_gets=None, field_values=None,
        field_value_templates=None, unique_conditional=None
    ):
        self.unique_conditional = unique_conditional or {}
        self.field_value_gets = field_value_gets or {}
        self.field_value_templates = field_value_templates or {}
        self.field_values = field_values or {}

        for field_name in field_names:
            self.fields.append(field_name)

            field = self.model._meta.get_field(field_name=field_name)

            if isinstance(field, models.fields.reverse_related.OneToOneRel):
                self.fields_related_one_to_one.append(field_name)
            elif isinstance(field, models.fields.reverse_related.ManyToOneRel):
                self.fields_reverse_related.append(field_name)
            elif isinstance(field, models.fields.related.ForeignKey):
                self.fields_foreign_keys.append(field_name)
            elif isinstance(field, contenttypes.fields.GenericRelation):
                self.fields_generic_related.append(field_name)
            elif isinstance(field, contenttypes.fields.GenericForeignKey):
                self.fields_foreign_keys.append(field_name)
            elif isinstance(field, models.fields.related.ManyToManyField):
                self.fields_many_to_many.append(field_name)
            elif isinstance(field, models.fields.reverse_related.ManyToManyRel):
                self.fields_many_to_many_reverse_related.append(field.related_name)
            else:
                if field.unique:
                    self.fields_unique.append(field_name)
                else:
                    self.fields_copy.append(field_name)

    def copy(self, instance, values=None):
        values = values or {}

        if self.excludes:
            if self.model._meta.default_manager.filter(pk=instance.pk, **self.excludes).exists():
                return

        if isinstance(self.model, mptt.models.MPTTModelBase):
            node_map = {instance.pk: None}
            self.field_value_templates = {
                'parent_id': '{parent_id}'
            }

            for source_node in instance.get_descendants(include_self=True):
                if source_node.parent:
                    values['parent_id'] = node_map[source_node.parent.pk]
                else:
                    values['parent_id'] = ''

                new_node = self._copy(
                    instance=source_node, values=values,
                    _get_or_create=self.extra_kwargs.get('get_or_create', False)
                )
                if not values['parent_id']:
                    result = new_node
                node_map[source_node.pk] = new_node.pk

            return result
        else:
            return self._copy(instance=instance, values=values)

    def _copy(self, instance, values=None, _get_or_create=False):
        context = {'instance': instance}
        context.update(values)

        new_model_dictionary = {}

        # Static values
        for field, value in self.field_values.items():
            new_model_dictionary[field] = value

        # Static values templates
        for field, value in self.field_value_templates.items():
            result = value.format(**context) or None
            new_model_dictionary[field] = result

        # Base fields whose values are copied
        for field in self.fields_copy:
            value = values.get(field, getattr(instance, field))

            value = self._evaluate_field_get_for_field(
                field=field, instance=instance, value=value, values=values
            )
            new_model_dictionary[field] = value

        # Base fields with unique values
        for field in self.fields_unique:
            base_value = getattr(instance, field)
            counter = 1

            while True:
                value = '{}_{}'.format(base_value, counter)
                if not self.model._meta.default_manager.filter(**{field: value}).exists():
                    break

                counter = counter + 1

            value = self._evaluate_field_get_for_field(
                field=field, instance=instance, value=value, values=values
            )
            new_model_dictionary[field] = value

        # Foreign keys
        for field in self.fields_foreign_keys:
            value = values.get(field, getattr(instance, field))

            value = self._evaluate_field_get_for_field(
                field=field, instance=instance, value=value, values=values
            )
            new_model_dictionary[field] = value

        # Fields that are given an unique value if a condition is met
        for field in self.unique_conditional:
            if self.unique_conditional[field](
                instance=instance, new_instance_dictionary=new_model_dictionary
            ):
                base_value = getattr(instance, field)
                counter = 1

                while True:
                    value = '{}_{}'.format(base_value, counter)
                    if not self.model._meta.default_manager.filter(**{field: value}).exists():
                        break

                    counter = counter + 1

                value = self._evaluate_field_get_for_field(
                    field=field, instance=instance, value=value, values=values
                )
                new_model_dictionary[field] = value

        if _get_or_create:
            new_instance, created = self.model._meta.default_manager.get_or_create(
                **new_model_dictionary
            )
        else:
            new_instance = self.model(**new_model_dictionary)
            new_instance.save()

        # Many to many fields added after instance creation
        for field in self.fields_many_to_many:
            getattr(new_instance, field).set(getattr(instance, field).all())

        # Many to many reverse related fields added after instance creation
        for field in self.fields_many_to_many_reverse_related:
            getattr(new_instance, field).set(getattr(instance, field).all())

        # Reverse related
        for field in self.fields_reverse_related:
            related_field = self.model._meta.get_field(field_name=field)
            related_field_name = related_field.field.name

            for related_instance in getattr(instance, field).all():
                values.update({related_field_name: new_instance})
                related_instance.copy_instance(
                    values=values
                )

        # Reverse related one to one
        for field in self.fields_related_one_to_one:
            related_field = self.model._meta.get_field(field_name=field)
            related_field_name = related_field.field.name

            getattr(instance, field).copy_instance(
                values={related_field_name: new_instance}
            )

        # Generic relations
        for field in self.fields_generic_related:
            related_field = self.model._meta.get_field(field_name=field)
            related_field_name = 'content_object'

            for related_instance in getattr(instance, field).all():
                related_instance.copy_instance(
                    values={related_field_name: new_instance}
                )

        return new_instance

    def get_fields_verbose_names(self):
        result = []

        for field_name in self.fields:
            field = self.model._meta.get_field(field_name=field_name)

            verbose_name = getattr(field, 'verbose_name', None)

            if not verbose_name and field.related_model:
                verbose_name = field.related_model._meta.verbose_name

            result.append(verbose_name)

        return result

    def test_condition(self, instance):
        if self.condition:
            return self.condition(instance=instance)
        else:
            return True


class MissingItem:
    _registry = []

    @classmethod
    def get_all(cls):
        return cls._registry

    @classmethod
    def get_missing(cls):
        result = []
        for item in cls.get_all():
            if item.condition():
                result.append(item)
        return result

    def __init__(self, label, condition, description, view):
        self.label = label
        self.condition = condition
        self.description = description
        self.view = view
        self.__class__._registry.append(self)


class PropertyHelper:
    """
    Makes adding fields using __class__.add_to_class easier.
    Each subclass must implement the `constructor` and the `get_result`
    method.
    """
    @staticmethod
    @property
    def constructor(source_object):
        return PropertyHelper(source_object)

    def __init__(self, instance):
        self.instance = instance

    def __getattr__(self, name):
        return self.get_result(name=name)

    def get_result(self, name):
        """
        The method that produces the actual result. Must be implemented
        by each subclass.
        """
        raise NotImplementedError


class ModelAttribute:
    _class_registry = []
    _model_registry = {}

    @classmethod
    def get_all_choices_for(cls, model):
        result = []

        for klass in cls._class_registry:
            klass_choices = klass.get_choices_for(model=model)
            if klass_choices:
                result.append(
                    (klass.class_label, klass_choices)
                )

        return result

    @classmethod
    def get_choices_for(cls, model):
        return sorted(
            [
                (entry.name, entry.get_display()) for entry in cls.get_for(model=model)
            ], key=lambda x: x[1]
        )

    @classmethod
    def get_for(cls, model):
        try:
            return cls._model_registry[cls.class_name][model]
        except KeyError:
            # We were passed a model instance, try again using the model of
            # the instance

            # If we are already in the model class, exit with an error
            if model.__class__ == models.base.ModelBase:
                return []

            return cls.get_for(model=type(model))

    @classmethod
    def register(cls, klass):
        cls._class_registry.append(klass)

    def __init__(self, model, name, label=None, description=None):
        self.model = model
        self.label = label
        self.name = name
        self.description = description
        self._model_registry.setdefault(self.class_name, {})
        self._model_registry[self.class_name].setdefault(model, [])
        self._model_registry[self.class_name][model].append(self)

    def get_display(self, show_name=False):
        if self.description:
            return '{} - {}'.format(
                self.name if show_name else self.label, self.description
            )
        else:
            return force_text(s=self.name if show_name else self.label)


class ModelProperty(ModelAttribute):
    class_label = _('Model properties')
    class_name = 'property'


class ModelField(ModelAttribute):
    class_label = _('Model fields')
    class_name = 'field'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._final_model_verbose_name = None

        if not self.label:
            self.label = self.get_field_attribute(
                attribute='verbose_name'
            )
            if self.label != self._final_model_verbose_name:
                self.label = '{}, {}'.format(
                    self._final_model_verbose_name, self.label
                )

        if not self.description:
            self.description = self.get_field_attribute(
                attribute='help_text'
            )

    def get_field_attribute(self, attribute, model=None, field_name=None):
        if not model:
            model = self.model

        if not field_name:
            field_name = self.name

        parts = field_name.split('__')
        if len(parts) > 1:
            return self.get_field_attribute(
                model=model._meta.get_field(parts[0]).related_model,
                field_name='__'.join(parts[1:]), attribute=attribute
            )
        else:
            self._final_model_verbose_name = model._meta.verbose_name
            return getattr(
                model._meta.get_field(field_name=field_name),
                attribute
            )


class ModelFieldRelated(ModelField):
    class_label = _('Model related fields')
    class_name = 'related_field'


class ModelReverseField(ModelField):
    class_label = _('Model reverse fields')
    class_name = 'reverse_field'

    def __init__(self, *args, **kwargs):
        super(ModelField, self).__init__(*args, **kwargs)
        self._final_model_verbose_name = None

        if not self.label:
            self.label = self.get_field_attribute(
                attribute='verbose_name_plural'
            )

    def get_field_attribute(self, attribute, model=None, field_name=None):
        if not model:
            model = self.model

        if not field_name:
            field_name = self.name

        return getattr(
            model._meta.get_field(field_name=field_name).related_model._meta,
            attribute
        )


class ModelQueryFields:
    _registry = {}

    @classmethod
    def get(cls, model):
        try:
            return cls._registry[model]
        except KeyError:
            ModelQueryFields(model=model)
            return cls.get(model=model)

    def __init__(self, model):
        self.model = model
        self.select_related_fields = []
        self.prefetch_related_fields = []
        self.__class__._registry[model] = self

    def add_select_related_field(self, field_name):
        if field_name in self.select_related_fields:
            raise ImproperlyConfigured(
                '"{}" model already has a "{}" query select related field.'.format(
                    self.model, field_name
                )
            )
        self.select_related_fields.append(field_name)

    def add_prefetch_related_field(self, field_name):
        if field_name in self.prefetch_related_fields:
            raise ImproperlyConfigured(
                '"{}" model already has a "{}" query prefetch related field.'.format(
                    self.model, field_name
                )
            )
        self.prefetch_related_fields.append(field_name)

    def get_queryset(self, manager_name=None):
        if manager_name:
            manager = getattr(self.model, manager_name)
        else:
            manager = self.model._meta.default_manager

        queryset = manager.all()

        if self.select_related_fields:
            queryset = queryset.select_related(*self.select_related_fields)

        if self.prefetch_related_fields:
            queryset = queryset.prefetch_related(*self.prefetch_related_fields)

        return queryset


class QuerysetParametersSerializer:
    @staticmethod
    def decompose(_model, _method_name, _manager_name=None, **kwargs):
        ContentType = apps.get_model(
            app_label='contenttypes', model_name='ContentType'
        )

        _manager_name = _manager_name or _model._meta.default_manager.name

        serialized_kwargs = []
        for name, value in kwargs.items():
            try:
                content_type = ContentType.objects.get_for_model(model=value)
            except AttributeError:
                """The value is not a model instance, pass it as-is."""
                serialized_kwargs.append(
                    {
                        'name': name,
                        'value': value
                    }
                )
            else:
                serialized_kwargs.append(
                    {
                        'name': name,
                        'content_type_id': content_type.pk,
                        'object_id': value.pk
                    }
                )

        return {
            'model_content_type_id': ContentType.objects.get_for_model(
                model=_model
            ).pk,
            'manager_name': _manager_name,
            'method_name': _method_name,
            'kwargs': serialized_kwargs
        }

    @staticmethod
    def rebuild(decomposed_queryset):
        ContentType = apps.get_model(
            app_label='contenttypes', model_name='ContentType'
        )

        model = ContentType.objects.get(
            pk=decomposed_queryset['model_content_type_id']
        ).model_class()

        queryset = getattr(model, decomposed_queryset['manager_name'])

        kwargs = {}

        for parameter in decomposed_queryset.get('kwargs', ()):
            if 'content_type_id' in parameter:
                content_type = ContentType.objects.get(
                    pk=parameter['content_type_id']
                )
                value = content_type.get_object_for_this_type(
                    pk=parameter['object_id']
                )
            else:
                value = parameter['value']

            kwargs[parameter['name']] = value

        return getattr(
            queryset, decomposed_queryset['method_name']
        )(**kwargs)


ModelAttribute.register(klass=ModelProperty)
ModelAttribute.register(klass=ModelField)
ModelAttribute.register(klass=ModelFieldRelated)
ModelAttribute.register(klass=ModelReverseField)



import hashlib

from django.template import Context, Engine, Template as DjangoTemplate
from django.template.response import TemplateResponse
from django.urls import reverse

from mayan.apps.common.settings import setting_home_view


class AJAXTemplate:
    _registry = {}

    @classmethod
    def all(cls, rendered=False, request=None):
        if not rendered:
            return cls._registry.values()
        else:
            result = []
            for template in cls._registry.values():
                result.append(template.render(request=request))
            return result

    @classmethod
    def get(cls, name):
        return cls._registry[name]

    def __init__(self, name, template_name):
        self.name = name
        self.template_name = template_name
        self.__class__._registry[name] = self

    def get_absolute_url(self):
        return reverse(
            kwargs={'name': self.name}, viewname='rest_api:template-detail'
        )

    def render(self, request):
        context = {
            'home_view': setting_home_view.value,
        }
        result = TemplateResponse(
            request=request,
            template=self.template_name,
            context=context,
        ).render()

        # Calculate the hash of the bytes version but return the unicode
        # version
        self.html = result.rendered_content.replace('\n', '')
        self.hex_hash = hashlib.sha256(result.content).hexdigest()
        return self


class Template:
    def __init__(self, template_string):
        engine = Engine(
            builtins=[
                'mathfilters.templatetags.mathfilters',
                'mayan.apps.templating.templatetags.templating_tags',
            ]
        )

        self._template = DjangoTemplate(
            engine=engine, template_string=template_string
        )

    def render(self, context=None):
        context_object = Context(dict_=context or {})

        return self._template.render(context=context_object)



import logging

from django.conf import settings
from django.urls import reverse_lazy
from django.utils import timezone, translation
from django.utils.translation import ugettext_lazy as _

from mayan.apps.views.generics import SimpleView, SingleObjectEditView

from .forms import LocaleProfileForm, LocaleProfileForm_view

logger = logging.getLogger(name=__name__)


class CurrentUserLocaleProfileDetailsView(SimpleView):
    template_name = 'appearance/generic_form.html'

    def get_extra_context(self, **kwargs):
        return {
            'form': LocaleProfileForm_view(
                instance=self.request.user.locale_profile
            ),
            'read_only': True,
            'title': _('Current user locale profile details'),
        }


class CurrentUserLocaleProfileEditView(SingleObjectEditView):
    extra_context = {
        'title': _('Edit current user locale profile details')
    }
    form_class = LocaleProfileForm
    post_action_redirect = reverse_lazy(
        viewname='locales:current_user_locale_profile_details'
    )

    def form_valid(self, form):
        form.save()

        timezone.activate(timezone=form.cleaned_data['timezone'])
        translation.activate(language=form.cleaned_data['language'])

        if hasattr(self.request, 'session'):
            self.request.session[
                translation.LANGUAGE_SESSION_KEY
            ] = form.cleaned_data['language']
            self.request.session[
                settings.TIMEZONE_SESSION_KEY
            ] = form.cleaned_data['timezone']
        else:
            self.request.set_cookie(
                settings.LANGUAGE_COOKIE_NAME, form.cleaned_data['language']
            )
            self.request.set_cookie(
                settings.TIMEZONE_COOKIE_NAME, form.cleaned_data['timezone']
            )

        return super().form_valid(form=form)

    def get_object(self):
        return self.request.user.locale_profile



MAYAN_PYPI_NAME = 'mayan-edms'

MESSAGE_NOT_LATEST = '''The version you are using is outdated. The latest
version is %s'''
MESSAGE_UNKNOWN_VERSION = '''It is not possible to determine the latest
version available.'''
MESSAGE_UNEXPECTED_ERROR = '''Unexpected error trying to determine the
latest version available. Make sure your installation has a connection to
the internet; %s'''
MESSAGE_UP_TO_DATE = 'Your version is up-to-date.'

PYPI_URL = 'https://pypi.org'



import logging

from django.apps import apps

from mayan.apps.lock_manager.exceptions import LockError
from mayan.celery import app

from .literals import TASK_ASSET_IMAGE_GENERATE_RETRY_DELAY

logger = logging.getLogger(name=__name__)


@app.task(
    bind=True, default_retry_delay=TASK_ASSET_IMAGE_GENERATE_RETRY_DELAY
)
def task_asset_image_generate(self, asset_id):
    Asset = apps.get_model(
        app_label='converter', model_name='Asset'
    )

    asset = Asset.objects.get(pk=asset_id)

    try:
        return asset.generate_image()
    except LockError as exception:
        logger.warning(
            'LockError during attempt to generate asset "%s" image. '
            'Retrying.', asset.internal_name
        )
        raise self.retry(exc=exception)
